# -*- coding: utf-8 -*-
u"""

MVC scheme Model element to access Users data required to Login.

Class LoginModel:\n\t Provides public methods to access Users data

"""

import wx.lib.pubsub as pubsub
from Synergy import Model
from Synergy import Exceptions
import string
import types

    
class KorespondencjaModel(Model.Model):
    def __init__(self,*args,**kwargs):
        """Creates Model object instance.
        
        Keywords arguments: 
        cursor -- a DB API2 Cursor object 

        """
        super(KorespondencjaModel,self).__init__(*args,**kwargs)
	
        #self = self._models['Korespondencja']

# - Table Creation Fields ----------------------------------------------------------

    def GetTables(self):
        """ Returns list with tables names to be created during system initialization

        """
        return ['mail']


    def _createTable_mail_postgres(self):
        """ Returns SQL query to create clients table with PostgreSQL database engine

        """        
        return """CREATE TABLE mail(
                            mid SERIAL,
                            nr INT NOT NULL,
                            m_date DATE NOT NULL DEFAULT current_date,
                            from_to VARCHAR(128) NOT NULL,
                            address VARCHAR(256),
			    signature VARCHAR(128),
                            topic VARCHAR(256) NOT NULL,
                            incoming BOOLEAN,
                            polecony BOOLEAN,
                            ZPO BOOLEAN,
                            priorytetowy BOOLEAN,
                            CONSTRAINT uinque_mail_identifier PRIMARY KEY (mid)
                            ) WITH OIDS"""

# - Getters ------------------------------------------------------------

#    def GetMail(self, incoming = True):
 #       query = 'SELECT mid, m_date, topic FROM mail WHERE incoming = %s'
  #      self._cursor.execute(query, (incoming,))
   #     return self._cursor.fetchall()

    def GetMailColumnHeaders(self,db_columns = False):
	    if db_columns:
		    return ('nr','m_date','from_to','topic')
	    else:
		    return ('Nr', 'Data', 'Korespondent', u'Treść',)

    def GetLastMailNumber(self):
        query="SELECT max(nr)+1 FROM mail"

        try:
            self._cursor.execute(query)
            r=self._cursor.fetchone()
            if r[0]:
                return r[0]
            else: #ustawiam offset... wiem, że do dupy tak ale póki nie ma konfiguratora..
                return 1
        except Exception, error:
            print error


    def GetKorespondenci(self):
	query="SELECT DISTINCT from_to FROM mail"
	self._cursor.execute(query)
	from_to=self._cursor.fetchall()
	r_list=[]
	for each in from_to:
		r_list.append(each[0].decode('utf-8'))
	return r_list

    def GetMail(self, inc=None,pol = None, zpo = None, pri = None, start = None, stop = None, search = '',item=None):
        args=locals()
        args.pop('self')
        
        #print args
        myregexp= '.*'+search+'.*'
        queries=("SELECT COUNT(*) FROM mail ",
                 "SELECT mid, incoming,nr, m_date, from_to, topic, signature, address, polecony, ZPO, priorytetowy FROM mail ")

	column_names = ['mid','incoming','nr','m_date','from_to','topic','signature','address','polecony','ZPO','priorytetowy']
        if item==None:
		query=queries[0]
	else:
		query=queries[1]
        
        query_args=[]

        # Query conditions -----------------------------------------------
        query_conditions=[]        
        conditions={'start':('m_date>=%s', (start,)),
                    'stop':('m_date <= %s', (stop,)),
                    'search':('(from_to ~* %s OR topic ~* %s) ', (search,search)),
                    'pol':('polecony=%s', (pol,)),
                    'zpo':('ZPO=%s', (zpo,)),
                    'pri':('priorytetowy=%s',(pri,)),
                    'inc':('incoming=%s',(inc,))
                    }

        #args,posargs=arguments()
        for arg in conditions.keys():
            if args[arg] not in (None,''):
                query_conditions.append(conditions[arg][0])
                query_args.extend(conditions[arg][1][:])
        if query_conditions:
            query+='WHERE '+string.join(query_conditions,' AND ')


        # Query postfix  -----------------------------------------------
        query_postfixes=[]        
        postfixes={'item':(' ORDER BY nr LIMIT 1 OFFSET %s', (item,))
                    }

        #args,posargs=arguments()
        for arg in postfixes.keys():
            if args[arg] not in (None,''):
                query_postfixes.append(postfixes[arg][0])
                query_args.extend(postfixes[arg][1][:])
        if query_postfixes:
            query+=string.join(query_postfixes,'')
            

        self._cursor.execute(query,tuple(query_args))
        
        result=self._cursor.fetchone()

        if item==None:
		# Zwracam ilość poczty
		return result[0]
        else:
		# Musze przerobic na unicode dane dla Windowsa...
		unicoded_result={}
		for i,each in enumerate(result):
			unicoded_result[column_names[i]]= each.decode('utf-8') if type(each)==types.StringType else each

		return Mail(data = unicoded_result)

# - Setters ------------------------------------------------------------

    def InsertMail(self, mail):

	data = mail.GetData()
	data.pop('mid')
        try:
            self._cursor.execute(self.ParseInsertQuery(data,'mail'),data.values())
            self._db.commit()         
            pubsub.Publisher().sendMessage("PostBook.Model.Events", data=('NewMail',[]))        
        except Exception, error:            
            self._db.rollback()            
            raise Exceptions.DatabaseError(error.message)


    def UpdateMail(self,mail):
        """Update Mail data
        
        """
	data = mail.GetData()
	where={'mid':data.pop('mid')}
        try:
		
            values=data.values()
            values.append(where.values()[0])
            self._cursor.execute(self.ParseUpdateQuery(data,'mail',where),values)
            self._db.commit()
            pubsub.Publisher().sendMessage("PostBook.Model.Events", data=('MailUpdated',mail))        
        except Exception, error:            
            self._db.rollback()            
            raise Exceptions.DatabaseError(error.message)


class Mail(object):

	_model = KorespondencjaModel()
	
	def __init__(self, data = None,*args,**kwargs):
		super(Mail,self).__init__(*args,**kwargs)
		self._data = {
			'mid': None,
			'nr': None,
			'm_date': None,
			'from_to': None,
			'address': None,
			'signature': None,
			'topic':None,
			'incoming':None,
			'polecony':None,
			'ZPO':None,
			'priorytetowy':None}

		if data != None:
			for key in self._data.keys():
				if data.has_key(key):
					self._data[key] = data[key]
	def GetData(self):
		return self._data

	def SetIncoming(self, incoming):
		self._data['incoming'] = incoming

	def Save(self):
		"""Save the Mail in database.

		"""		
		if self._data['mid'] !=None:
			print 'Robie update!!'			
			self._model.UpdateMail(self)
		elif self._data['mid'] == None:			
			self._model.InsertMail(self)



	
