# -*- coding: utf-8 -*-
import wx
import wx.lib.pubsub as pubsub

from Synergy.Components.Share.Share import CommonPanel

from Synergy.Visual import VisualToolBar
from Synergy.Components.Share import Share

import Images
from wx.lib.art import img2pyartprov

from DataFields import DateCtrlComboPopup as DateCtrl, MailTypesWithBorder as mt, MailVirtualList, MailPanel,EVT_MAILTYPE
import wx.lib.flatnotebook as fnb


class ToolBarMain(VisualToolBar):
	def __init__(self,*args,**kwargs):
		super(ToolBarMain,self).__init__(*args,**kwargs)	
		self._addAP()
		
	def _addAP(self):		
		ToolbarArtProvider = img2pyartprov.Img2PyArtProvider(Images,artIdPrefix='wx.ART_POSTBOOK_')
		wx.ArtProvider.Push(ToolbarArtProvider)		
		
	def _toolsData(self,toolbar):
		in_bmp =  wx.ArtProvider.GetBitmap('wx.ART_POSTBOOK_MAIL_IN', wx.ART_OTHER, self.tsize)		
		out_bmp =  wx.ArtProvider.GetBitmap('wx.ART_POSTBOOK_MAIL_OUT', wx.ART_OTHER, self.tsize)		
		
		# Label, Bitmap, Short Help, Long Helo, Event, Handler 
		data=(
                (u'Nowa poczta przychodząca',in_bmp,u'Dodaj pocztę przychodzącą',u'Dodaj pocztę przychodzącą',wx.EVT_TOOL,self.OnIn),
                (u'Nowa poczta wychodząca',out_bmp,u'Dodaj pocztę wychodzącą',u'Dodaj pocztę wychodzącą',wx.EVT_TOOL,self.OnOut),
		      (""),
		      )
		return data	

	def OnIn(self,event):
		data=('NewMail',['In'])
		pubsub.Publisher().sendMessage("PostBook.Events.Dialog",data=data)
		
	def OnOut(self,event):
		data=('NewMail',['Out'])
		pubsub.Publisher().sendMessage("PostBook.Events.Dialog",data=data)
		

class RightNotebook(fnb.FlatNotebook):
	def __init__(self,*args,**kwargs):
		super(RightNotebook,self).__init__(style=fnb.FNB_NODRAG|fnb.FNB_NO_X_BUTTON,*args,**kwargs)

		self.inMail = MailPanel(self,-1, mail_type = True)
		self.outMail = MailPanel(self,-1,mail_type = False)

		self.AddPage(self.inMail, u'Poczta przychodząca')
		self.AddPage(self.outMail, u'Poczta wychodząca')
		self.SetSelection(0)
		self.Bind(wx.EVT_NOTEBOOK_PAGE_CHANGED,self.OnPageChanged)

	def OnPageChanged(self,evt):
		self.inMail.Refresh()
		self.outMail.Refresh()

	def ShowData(self, pol = None, zpo = None, pri = None, start = None, stop = None, search = ''):
		if start=='':
		    start=None
		if stop=='':
		    stop=None

		self.inMail.ShowData(pol = pol, zpo = zpo, pri = pri, start = start, stop = stop, search = search)
		self.outMail.ShowData(pol = pol, zpo = zpo, pri = pri, start = start, stop = stop, search = search)
		self.inMail.Refresh()
		self.outMail.Refresh()

class DataPanelMail(Share.CommonPanel):
	def __init__(self,parent,*args,**kwargs):
		super(DataPanelMail,self).__init__(parent=parent,name='Korespondencja',dbAccessColumn='view_cases',*args,**kwargs)
		self._data={}
		self.tButtonState=False
		self._filter=None
		self._init_ctrls(self)
		self._Subscribe()
		self.nazwa=''

	def _Subscribe(self):		
		pubsub.Publisher().subscribe(self.OnEvent, "Model.Events")
		pubsub.Publisher().subscribe(self.OnEvent, "PostBook.Model.Events")
		

	def OnEvent(self,message):
		if hasattr(self,message.data[0]):
			getattr(self,message.data[0])(*message.data[1:])

	def NewMail(self,*args):
		self.RefreshMail()

	def MailUpdated(self,*args):
		self.RefreshMail()

	#def ModelReady(self,*args):
	#	self.Rnotes.SetModel(self._model)

	def _createDate(self,sizer):

		FGSizer=wx.FlexGridSizer(cols=2,hgap=5,vgap=5)
		FGSizer.AddGrowableCol(1,1)
		sizer.Add(FGSizer,1,flag=wx.EXPAND|wx.ALL,border=10)
		
		self._ctrls['from']= wx.combo.ComboCtrl(self, size=(-1,-1),style=wx.TE_READONLY)
		tcpA = DateCtrl()
		self._ctrls['from'].SetPopupControl(tcpA)

		date=wx.DateTime()
		date.SetToCurrent()
		month=date.GetCurrentMonth()

		currDate=date.GetDateOnly()
		currDate.SetDay(1)
		nextWeek=date.SetToLastMonthDay()
		currDate="%s-%s-%s"%(currDate.GetYear(),currDate.GetMonth()+1,currDate.GetDay())
		nextWeek="%s-%s-%s"%(nextWeek.GetYear(),nextWeek.GetMonth()+1,nextWeek.GetDay())
		
		self._ctrls['from'].SetValue(currDate)
		label=wx.StaticText(self,-1,unicode("Od",'utf-8'))
		FGSizer.Add(label,0,wx.ALIGN_CENTER)
		FGSizer.Add(self._ctrls['from'],1,wx.GROW)



		self._ctrls['to'] = wx.combo.ComboCtrl(self, size=(-1,-1),style=wx.TE_READONLY)
		tcpB = DateCtrl()
		self._ctrls['to'].SetPopupControl(tcpB)
		self._ctrls['to'].SetValue(nextWeek)
		#self.cal.Bind(wx.EVT_LEFT_DOWN, self.OnLeftDown)
		#self.Bind(wx.EVT_LEFT_DOWN,self.OnDateChange,self._ctrls['to'])
		label=wx.StaticText(self,-1,unicode("Do",'utf-8'))
		FGSizer.Add(label,0,wx.ALIGN_CENTER)
		FGSizer.Add(self._ctrls['to'],1,wx.GROW)
		#but=wx.Button(self,-1,u"Pokaż")
		#FGSizer.Add(but,1,wx.GROW)
		#self.Bind(wx.EVT_BUTTON,self.OnDateChange,but)

		self.Bind(wx.calendar.EVT_CALENDAR,self.OnDateChange,tcpA.cal)
		self.Bind(wx.calendar.EVT_CALENDAR,self.OnDateChange,tcpB.cal)




	def OnDateChange(self,event):
		start_date=self._ctrls['from'].GetValue()
		stop_date=self._ctrls['to'].GetValue()
		self.Rnotes.ShowData(start=start_date,stop=stop_date)

		
	def _init_date(self,sizer):
		sizerStat=wx.StaticBoxSizer(wx.StaticBox(self,1,u'Korespondencja z okresu') ,wx.VERTICAL)
		sizer.AddSizer(sizerStat,0,wx.GROW|wx.LEFT|wx.TOP, border=10)
		self._createDate(sizerStat)


	def _init_actions(self,sizer):

		self.Rnotes=RightNotebook(self,-1)

		sizer.Add(self.Rnotes,1,wx.GROW|wx.TOP|wx.LEFT|wx.RIGHT, border=10)


	def _init_leader(self,sizer):

		sizerStat=wx.StaticBoxSizer(wx.StaticBox(self,1,u'Filtr korespondencji') ,wx.VERTICAL)
		sizer.AddSizer(sizerStat,0,wx.GROW|wx.LEFT|wx.TOP, border=10)
 		self.search=wx.SearchCtrl(self,-1)
		self.search.SetDescriptiveText(u'Wpisz słowo')
		sizerStat.Add(self.search,1,wx.GROW)
 		self.Bind(wx.EVT_TEXT,self.OnSearch,self.search)

		self.mTypes=mt(self)
		self.mTypes.Enable(False)
		sizer.Add(self.mTypes,0,wx.GROW|wx.LEFT|wx.TOP, border=10)
		self.Bind(EVT_MAILTYPE, self.OnMT)
		#self.Bind(wx.EVT_CHECKBOX, self.OnMT)
		

		self.tButton = wx.ToggleButton(self, -1, u'Filtruj względem typów')
		self.Bind(wx.EVT_TOGGLEBUTTON, self.OnToggle, self.tButton)
		sizer.Add(self.tButton,0,wx.GROW|wx.LEFT|wx.TOP, border=10)
		
		#evt=wx.CommandEvent(wx.wxEVT_COMMAND_CHOICE_SELECTED,self.seller.GetId())
		#self.GetEventHandler().ProcessEvent(evt)

	def OnMT(self,evt):
		#self.mTypes.OnZPO(evt)
		self.RefreshMail()
		#evt.Skip()

	def OnToggle(self,evt):
		self.tButtonState = self.tButton.GetValue()
		self.mTypes.Enable(self.tButtonState)
		self.RefreshMail()
		
	def OnSearch(self,evt):
		self.nazwa=self.search.GetValue()
		self.RefreshMail()



	def RefreshMail(self):
		start_date=self._ctrls['from'].GetValue()
		stop_date=self._ctrls['to'].GetValue()
		if self.tButtonState:
			mTyps=self.mTypes.GetData()
		else:
			mTyps={'polecony':None,
			       'priorytetowy':None,
			       'ZPO':None}
		
		self.Rnotes.ShowData(pol = mTyps['polecony'], pri = mTyps['priorytetowy'], zpo = mTyps['ZPO'],
				     start=start_date,stop=stop_date, search = self.nazwa)

		#self.Rnotes.RefreshAll()

	
	def _init_ctrls(self, parent):			

		horSizer=wx.BoxSizer(wx.HORIZONTAL)
		vertSizer=wx.BoxSizer(wx.VERTICAL)
		vertSizerTerms=wx.BoxSizer(wx.VERTICAL)
		horSizer.AddSizer(vertSizer,0,wx.GROW)
		horSizer.AddSizer(vertSizerTerms,0,wx.GROW)
		self._ctrls={}

		self._init_leader(vertSizer)
		#self._init_info(vertSizer)
		self._init_actions(horSizer)
		self._init_date(vertSizer)
		#self._init_stats(vertSizer)
                
		#self._createList(vertSizer,self)
		self.SetSizer(horSizer)
		self.SetAutoLayout(True)


